/*==============================================================================*/
/*  Copyright (C) 2009-2018, Heartland Data inc. All Rights Reserved.           */
/*                                                                              */
/*  Title  :   GPIO 4bit Driver                                                 */
/*  EventID:   Via the driver                                                   */
/*  AppVer :   DT10 Ver10.XX`@                                                */
/*  FileID :   12b                                                              */
/*  Version:   1.3                                                              */
/*  Author :   HLDC                                                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Please customize the code for your environment.                             */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Header for Port Control                                             */
/*==============================================================================*/
#include "Common.h"
// #include <itron.h>
// #include <kernel.h>

/*==============================================================================*/
/*  Macro:  DT_UINT                                                             */
/*  Desc:   Please change Test Point argument type for DT10 Project setting.    */
/*==============================================================================*/
#define	DT_UINT unsigned int

/*==============================================================================*/
/*  Macro:  DT_INLINE                                                           */
/*  Desc:   Please use "static" instead of "inline" if "inline" cannot be used. */
/*==============================================================================*/
#define	DT_INLINE	inline
/* #define	DT_INLINE	static */

/*==============================================================================*/
/*  Macro:  DT_ADD_EVENT_INFO                                                   */
/*  Desc:   Please set 1 when you add Event Value to Test Point.                */
/*==============================================================================*/
#define	DT_ADD_EVENT_INFO 0

/*==============================================================================*/
/*  Func:   _TP_BusPortInit                                                     */
/*  Desc:   Please describe the code to initializes ports.                      */
/*==============================================================================*/
static void _TP_BusPortInit(void)
{
	/* ex)	
			PA.DIR = 0x3f;
			PD.DIR = 0x3f;
			PA.word = 0x3f;
			PD.word = 0x3f;	
	*/
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOA );
	SysCtlPeripheralEnable( SYSCTL_PERIPH_GPIOD );
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_2 );		/*	CLK	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTD_BASE, GPIO_PIN_3 );		/*	CS	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_2 );		/*	DAT0	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_3 );		/*	DAT1	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_4 );		/*	DAT2	*/
	GPIOPinTypeGPIOOutput( GPIO_PORTA_BASE, GPIO_PIN_5 );		/*	DAT3	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2);
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, GPIO_PIN_3);
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_2, GPIO_PIN_2 );
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_3, GPIO_PIN_3 );
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_4, GPIO_PIN_4 );
	GPIOPinWrite( GPIO_PORTA_BASE, GPIO_PIN_5, GPIO_PIN_5 );
}

/*==============================================================================*/
/*  Func:   portReverseCLK                                                      */
/*  Desc:   Please describe the code to reverse CLK Port.                       */
/*==============================================================================*/
DT_INLINE void portReverseCLK(void)
{
	/* ex)
		PA.bit.b0 ^= 1;
	*/
	if( GPIOPinRead( GPIO_PORTD_BASE, GPIO_PIN_2 ) ){
		GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, 0 );
	}
	else{
		GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, GPIO_PIN_2 );
	}
}

/*==============================================================================*/
/*  Func:   portSetCLK                                                          */
/*  Desc:   Please describe the code to set the value to CLK Port.              */
/*==============================================================================*/
DT_INLINE void portSetCLK( DT_UINT dat)
{
	/* ex)
		PA.bit.b0 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_2, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   portSetCS                                                           */
/*  Desc:   Please describe the code to set the value to CS Port.               */
/*==============================================================================*/
DT_INLINE void portSetCS(DT_UINT dat)
{
	/* ex)
		PA.bit.b1 = dat?1:0;
	*/
	GPIOPinWrite( GPIO_PORTD_BASE, GPIO_PIN_3, dat?0xff:0x00 );
}

/*==============================================================================*/
/*  Func:   portSetDATA                                                         */
/*  Desc:   Please describe the code to set the value to DATA Port              */
/*==============================================================================*/
DT_INLINE void portSetDATA(DT_UINT dat)
{
	/* ex)
	 PA.word = (PA.word & 0xff0f) | ((dat & 0x0f) << 4);
	*/
	GPIOPinWrite( GPIO_PORTA_BASE, (GPIO_PIN_2|GPIO_PIN_3|GPIO_PIN_4|GPIO_PIN_5), (dat & 0x0f) << 2 );
}

/*==============================================================================*/
/*  Func:   enterCritical                                                       */
/*  Desc:   Please describe the code to enter Critical Section.                 */
/*==============================================================================*/
DT_INLINE void enterCritical(void)
{
	/* ex)
		DI();
	*/ 
	taskENTER_CRITICAL();
}

/*==============================================================================*/
/*  Func:   exitCritical                                                        */
/*  Desc:   Please describe the code to exit Critical Section.                  */
/*==============================================================================*/
DT_INLINE void exitCritical(void)
{
	/* ex)
		EI();
	*/ 
	taskEXIT_CRITICAL();
}

/*==============================================================================*/
/*  Func:   addEventInfo                                                        */
/*  Desc:   please describe the code to get target event                        */
/*==============================================================================*/
#if DT_ADD_EVENT_INFO
DT_INLINE DT_UINT addEventInfo(void)
{
	int event_id = 0;

	/* describe code to get target eventinformation */
	/* ex. uITron4.0 API	*/
	//ID task_id;
	//get_tid( &p_task_id );
	//event_id = (DT_UINT)task_id;
	return event_id;

}
#endif

/*==============================================================================*/
/*  Don't change the code from here as possible.                                */
/*==============================================================================*/

/*==============================================================================*/
/*  Desc:   Test Point Parametaer                                               */
/*==============================================================================*/
#define		DT_VARIABLE_BIT			0x02
#define		DT_TARGET_TIME_BIT		0x04
#define		DT_EVTTRG_BIT			0x08
#define		DT_VARIABLE_FAST_BIT	0x01
#define		DT_EVTTRG_FAST_BIT		0x02
#define		DT_VARIABLE_WRITE_BIT	0x01

/*==============================================================================*/
/*  Func:   portInit                                                            */
/*  Desc:   Initialize Port Function                                            */
/*==============================================================================*/
static int init = 0;
DT_INLINE void portInit(void)
{
	if( init == 0 ){
		_TP_BusPortInit();
		init = 1;
	}
}

/*==============================================================================*/
/*  Func:   _TP_Bus4BitOutDrv                                                   */
/*  Desc:   4bit Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_Bus4BitOutDrv( DT_UINT dat )
{
	portSetDATA( dat );
	portReverseCLK();
}

/*==============================================================================*/
/*  Func: _TP_BusOutDrv                                                         */
/*  Desc: Test Point Output Function                                            */
/*==============================================================================*/
DT_INLINE void _TP_BusOutDrv( DT_UINT addr, DT_UINT dat )
{
	portSetCS(0);
	_TP_Bus4BitOutDrv( dat >> 12 );
	_TP_Bus4BitOutDrv( dat >> 8 );
	_TP_Bus4BitOutDrv( dat >> 4 );
	_TP_Bus4BitOutDrv( dat );
	addr &= 0xfffff;
	if( addr >= 0x10000 ) _TP_Bus4BitOutDrv( addr >> 16 );
	if( addr >= 0x1000 ) _TP_Bus4BitOutDrv( addr >> 12 );
	if( addr >= 0x0100 ) _TP_Bus4BitOutDrv( addr >> 8 );
	if( addr >= 0x0010 ) _TP_Bus4BitOutDrv( addr >> 4 );
	_TP_Bus4BitOutDrv( addr );
	portSetCS(1);
}

/*==============================================================================*/
/*  Func:   _TP_BusOutByteDrv                                                   */
/*  Desc:   Byte Data Output Function                                           */
/*==============================================================================*/
DT_INLINE void _TP_BusOutByteDrv( DT_UINT dat )
{
	_TP_Bus4BitOutDrv( dat >> 4 );
	_TP_Bus4BitOutDrv( dat );
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutDrv                                                    */
/*  Desc:   Value Output Function                                               */
/*==============================================================================*/
DT_INLINE void _TP_MemoryOutDrv( unsigned char *p, DT_UINT size )
{
	if( size >= 256 ) size = 256;
	_TP_BusOutByteDrv( size );
	for( ; size != 0; --size, ++p ){
		_TP_BusOutByteDrv( *p );
	}
}

/*==============================================================================*/
/*  Func: _TP_BusOut                                                            */
/*  Desc: Called by Test Point                                                  */
/*==============================================================================*/
void _TP_BusOut( DT_UINT addr, DT_UINT dat )
{

#if DT_ADD_EVENT_INFO
	unsigned int event_id = 0;
#endif

	portInit();
	enterCritical();

#if DT_ADD_EVENT_INFO
	addr = addr | DT_EVTTRG_BIT;
#endif

	_TP_BusOutDrv( addr, dat );

#if DT_ADD_EVENT_INFO
	portSetCS(0);

	event_id = addEventInfo();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);

	portSetCS(1);
#endif
	exitCritical();
}

/*==============================================================================*/
/*  Func:   _TP_MemoryOutput                                                    */
/*  Desc:   Called by Variable Test Point                                       */
/*==============================================================================*/
void _TP_MemoryOutput( DT_UINT addr, DT_UINT dat, void *value, DT_UINT size )
{

#if DT_ADD_EVENT_INFO
	int event_id = 0;
#endif

	portInit();
	enterCritical();

#if DT_ADD_EVENT_INFO
	addr = addr | DT_EVTTRG_BIT;
#endif

	_TP_BusOutDrv( addr | DT_VARIABLE_BIT, dat );

	/* 2nd data output */
	 portSetCS(0);

#if DT_ADD_EVENT_INFO
	event_id = addEventInfo();
	_TP_BusOutByteDrv(event_id);
	_TP_BusOutByteDrv(event_id >> 8);
	_TP_BusOutByteDrv(event_id >> 16);
	_TP_BusOutByteDrv(event_id >> 24);
#endif

	/* output value */
	_TP_MemoryOutDrv( (unsigned char *)value, size );

	portSetCS(1);
	exitCritical();
}

